/*
 * GDIPlus.cpp
 * @desc GDI+Ɋւ鏈̂t@C
 * @auther K
 * $ld$
 * Copyright(c) 2005 K, All rights reserved
 */

#include <stdio.h>
#include <locale.h>
#include "GDIPlus.h"


/**
 *	\z
 *
 */
CGDIPlus::CGDIPlus( void )
{
	m_pImage		= NULL;
	m_pPropertyItem	= NULL;
	m_hGlobal		= NULL;
	m_pStream		= NULL;

	Initialize();
}

/**
 *	
 *
 */
CGDIPlus::~CGDIPlus( void )
{
	Uninitialize();
}

/**
 *	
 *
 */
void CGDIPlus::Initialize( void )
{
	Uninitialize();

	m_lNextImageTime	= 0;
	m_nFrameCount		= 0;
	m_nFramePosition	= 0;

	m_hDC				= NULL;
}

/**
 *	
 *
 */
void CGDIPlus::Uninitialize( void )
{
	if( m_pImage )
	{
		if( m_pPropertyItem )
		{
			m_pImage->RemovePropertyItem( m_pPropertyItem->id );
			::free( m_pPropertyItem );
			m_pPropertyItem = NULL;
		}
		delete m_pImage;
		m_pImage = NULL;
	}

	if( m_pStream )
	{
		m_pStream->Release();
		m_pStream = NULL;
	}

	if( m_pPropertyItem )
	{
		free( m_pPropertyItem );
		m_pPropertyItem = NULL;
	}

	if( m_hGlobal )
	{
		::GlobalFree( m_hGlobal );
		m_hGlobal = NULL;
	}
}

/**
 *	\摜C[Wݒit@Cǂݍ݁j
 *
 *	@param	pszFileName	: C[W̃t@C
 *	@result				: true/ false/s
 */
bool CGDIPlus::CreateImageFromFile( const char *pszFileName )
{
    WCHAR pwszData[MAX_PATH+1] = {0};

    // P[ftHgɐݒ
	::setlocale( LC_ALL, "" ); 

    // }`oCgChɕϊ
    ::mbstowcs( pwszData, pszFileName, ::strlen(pszFileName)+1 ); 

	Bitmap *pImage = Bitmap::FromFile( pwszData );

	// G[`FbN
	Status status = pImage->GetLastStatus();
	if( status == Ok )
	{
		if( m_pImage )
		{
			if( m_pPropertyItem )
			{
				m_pImage->RemovePropertyItem( m_pPropertyItem->id );
				::free( m_pPropertyItem );
				m_pPropertyItem = NULL;
			}
			delete m_pImage;
		}
		m_pImage = pImage;
	}
	else
	{
		delete pImage;
		pImage = NULL;

		return false;
	}

	// Assume that the image has a property item of type PropertyItemEquipMake.
	// Get the size of that property item.
	// vpeBACẽTCY擾
	unsigned int nSize = m_pImage->GetPropertyItemSize(PropertyTagFrameDelay);
	if( nSize != 0 )
	{
		// Allocate a buffer to receive the property item.
		// vpeBACẽobt@m
		m_pPropertyItem = (PropertyItem*)::malloc( nSize );

		// ҂ԃvpeB擾
		m_pImage->GetPropertyItem( PropertyTagFrameDelay, nSize, m_pPropertyItem );

		// t[fBV擾
		UINT nCount = m_pImage->GetFrameDimensionsCount();
		if( !nCount )
			return false;
		GUID* pDimensionIDs = new GUID[nCount];

		// Get the list of frame dimensions from the Image object.
		// t[fBV擾
		m_pImage->GetFrameDimensionsList( pDimensionIDs, nCount );

		// Get the number of frames in the first dimension.
		// ŏ̃fBVAt[擾
		m_nFrameCount = m_pImage->GetFrameCount( &pDimensionIDs[0] );

		delete []pDimensionIDs;
	}
	else
	{
		m_nFramePosition	= 0;
		m_nFrameCount		= 0;
	}

	return true;
}

/**
 *	\摜C[Wݒ
 *
 *	@param	pszData		: C[W̃obt@
 *	@param	dwSize		: obt@̑傫
 *	@result				: true/ false/s
 */
bool CGDIPlus::CreateImage( const unsigned char *pszData, DWORD dwSize )
{
	LPVOID		lpMemory	= NULL;

	if( m_hGlobal )
	{
		::GlobalFree( m_hGlobal );
	}
	m_hGlobal = ::GlobalAlloc( GMEM_MOVEABLE, dwSize );
	if( !m_hGlobal )
	{
		return false;
	}

	lpMemory = ::GlobalLock( m_hGlobal );
	if( !lpMemory )
	{
		::GlobalUnlock( m_hGlobal );
		return false;
	}
	::memcpy( lpMemory, pszData, dwSize );
	::GlobalUnlock( m_hGlobal );
	::CreateStreamOnHGlobal( m_hGlobal, TRUE, &m_pStream );

//	m_pImage = Image::FromStream( m_pStream );

	// Assume that the image has a property item of type PropertyItemEquipMake.
	// Get the size of that property item.
	// vpeBACẽTCY擾
	int nSize = m_pImage->GetPropertyItemSize(PropertyTagFrameDelay);
	if( nSize != 0 )
	{
		// Allocate a buffer to receive the property item.
		// vpeBACẽobt@m
		if( m_pPropertyItem )
			free( m_pPropertyItem );
		m_pPropertyItem = (PropertyItem*) malloc(nSize);

		// ҂ԃvpeB擾
		m_pImage->GetPropertyItem(PropertyTagFrameDelay, nSize, m_pPropertyItem);
	}

	return true;
}

/**
 *	摜ۑ
 *
 *	@param	pszFileName	: C[W̃t@C
 *	@param	nFramePosition: ۑt[
 *	@param	nSelectedFormat: ۑtH[}bg
 *	@result				: true/ false/s
 */
bool CGDIPlus::SaveImage( const char *pszFileName, int nFramePosition, int nSelectedFormat )
{
    WCHAR pwszData[MAX_PATH+1] = {0};

    // P[ftHgɐݒ
	::setlocale( LC_ALL, "" ); 

    // }`oCgChɕϊ
    ::mbstowcs( pwszData, pszFileName, ::strlen(pszFileName)+1 ); 

	UINT nEncoders;
	UINT nSize;
	GetImageEncodersSize( &nEncoders, &nSize );

    ImageCodecInfo encoders[0x100];
	GetImageEncoders( nEncoders, nSize, encoders );

	GUID pageGuid = FrameDimensionTime;
	m_pImage->SelectActiveFrame( &pageGuid, nFramePosition );

	m_pImage->Save( pwszData, &encoders[nSelectedFormat].Clsid );

	return true;
}

/**
 *	\ݒ
 *
 *	@param	hDC			: \HDC
 */
void CGDIPlus::SetDispParam( HDC hDC )
{
	m_hDC = hDC;
}

/**
 *	摜̉Ac擾
 *
 *	@param	pnWidth		: 
 *	@param	pnHeight	: c
 */
void CGDIPlus::GetImageSize( int *pnWidth, int *pnHeight )
{
	if( !m_pImage )
	{
		*pnWidth = 0;
		*pnHeight= 0;
		return;
	}

	*pnWidth = m_pImage->GetWidth();
	*pnHeight= m_pImage->GetHeight();
}

/**
 *	ǂݍ݂ɑΉĂ摜tH[}bg擾
 *
 *	@param	pszMessage	: tH[}bgꗗ
 */
void CGDIPlus::GetCanReadFormat( char *pszMessage )
{
	UINT nDecoders;
	UINT nSize;
	GetImageDecodersSize( &nDecoders, &nSize );

	ImageCodecInfo decoders[0x100];
	GetImageDecoders( nDecoders, nSize, decoders );

	int j = 0;
	for( UINT i = 0;i < nDecoders;i++ )
	{
		char pszData[0x100];
		::wcstombs( pszData, decoders[i].FormatDescription, ::wcslen(decoders[i].FormatDescription)+1 );
		::sprintf( &pszMessage[j], "%s\n", pszData );
		j += ::strlen( &pszMessage[j] );
	}
}

/**
 *	o͂ɑΉĂ摜tH[}bg擾
 *
 *	@param	pszFormat	: tH[}bg
 *	@param	pszExt		: gq
 *	@result				: tH[}bg
 */
int CGDIPlus::GetCanWriteFormat( char pszFormat[0x100][0x100], char pszExt[0x100][0x100] )
{
	UINT nEncoders;
	UINT nSize;
	GetImageEncodersSize( &nEncoders, &nSize );

	ImageCodecInfo encoders[0x100];
	GetImageEncoders( nEncoders, nSize, encoders );

	int j = 0;
	for( UINT i = 0;i < nEncoders;i++ )
	{
		char pszTmp1[0x100];
		::wcstombs( pszFormat[i], encoders[i].FormatDescription, ::wcslen(encoders[i].FormatDescription)+1 );
		::wcstombs( pszTmp1,      encoders[i].FilenameExtension, ::wcslen(encoders[i].FilenameExtension)+1 );

		// gqꍇ́Aŏ̂̂g
		char *pTmp2 = ::strchr( pszTmp1, ';' );
		if( pTmp2 )
			pszTmp1[pTmp2-pszTmp1] = '\0';
		// ŏ"*."΂
		::sprintf( pszExt[i], "%s", &pszTmp1[2] );
	}

	return nEncoders;
}

/**
 *	Aj[V
 *
 *	@result				: ̃t[\܂ł̎
 */
int CGDIPlus::DrawImage( void )
{
	GUID pageGuid = FrameDimensionTime;

	unsigned int nWidth = m_pImage->GetWidth();
	unsigned int nHeight= m_pImage->GetHeight();

	/*-------------------------------*/
	// t[摜ݒ
	/*-------------------------------*/
	m_pImage->SelectActiveFrame( &pageGuid, m_nFramePosition );

	/*-------------------------------*/
	// `
	/*-------------------------------*/
	if( m_hDC )
	{
		HBITMAP hBitmap = ::CreateCompatibleBitmap( m_hDC, nWidth, nHeight );
		::SelectObject( m_hDC, hBitmap );
		::DeleteObject( hBitmap );

		Graphics graphics( m_hDC );
		graphics.DrawImage( m_pImage, 0, 0, nWidth, nHeight );
	}

	/*-------------------------------*/
	// ̃t[܂ł̎Ԏ擾
	/*-------------------------------*/
	m_lNextImageTime = 0;
	if( m_pPropertyItem )
	{
		// GIFAj[V1/100sPʂȂ̂x10Ă
		m_lNextImageTime = ((long*) m_pPropertyItem->value)[m_nFramePosition] * 10;
	}

	return m_lNextImageTime;
}
